package com.sun.xml.internal.ws.handler;

import java.util.Collection;
import java.util.Iterator;
import com.sun.xml.internal.ws.api.message.AttachmentSet;
import javax.activation.DataHandler;
import com.sun.xml.internal.ws.api.message.Attachment;
import java.util.HashMap;
import java.util.Map;
import com.sun.xml.internal.ws.api.message.Packet;
import java.util.Set;
import javax.xml.ws.handler.MessageContext;

class MessageContextImpl implements MessageContext
{
    private final Set<String> handlerScopeProps;
    private final Packet packet;
    private final Map<String, Object> asMapIncludingInvocationProperties;
    
    public MessageContextImpl(Packet packet) {
        this.packet = packet;
        this.asMapIncludingInvocationProperties = packet.asMapIncludingInvocationProperties();
        this.handlerScopeProps = packet.getHandlerScopePropertyNames(false);
    }
    
    protected void updatePacket() {
        throw new UnsupportedOperationException("wrong call");
    }
    
    public void setScope(String name, MessageContext.Scope scope) {
        if (!this.containsKey(name)) {
            throw new IllegalArgumentException("Property " + name + " does not exist.");
        }
        if (scope == MessageContext.Scope.APPLICATION) {
            this.handlerScopeProps.remove(name);
        }
        else {
            this.handlerScopeProps.add(name);
        }
        return;
    }
    
    public MessageContext.Scope getScope(String name) {
        if (!this.containsKey(name)) {
            throw new IllegalArgumentException("Property " + name + " does not exist.");
        }
        if (this.handlerScopeProps.contains(name)) {
            return MessageContext.Scope.HANDLER;
        }
        return MessageContext.Scope.APPLICATION;
    }
    
    public int size() {
        return this.asMapIncludingInvocationProperties.size();
    }
    
    public boolean isEmpty() {
        return this.asMapIncludingInvocationProperties.isEmpty();
    }
    
    public boolean containsKey(Object key) {
        return this.asMapIncludingInvocationProperties.containsKey(key);
    }
    
    public boolean containsValue(Object value) {
        return this.asMapIncludingInvocationProperties.containsValue(value);
    }
    
    public Object put(String key, Object value) {
        if (!this.asMapIncludingInvocationProperties.containsKey(key)) {
            this.handlerScopeProps.add(key);
        }
        return this.asMapIncludingInvocationProperties.put(key, value);
    }
    
    public Object get(Object key) {
        if (key == null) {
            return null;
        }
        Object value = this.asMapIncludingInvocationProperties.get(key);
        if (key.equals("javax.xml.ws.binding.attachments.outbound") || key.equals("javax.xml.ws.binding.attachments.inbound")) {
            Map<String, DataHandler> atts = (Map)value;
            if (atts == null) {
                atts = new HashMap();
            }
            AttachmentSet attSet = this.packet.getMessage().getAttachments();
            for (Attachment att : attSet) {
                String cid = att.getContentId();
                if (cid.indexOf("@jaxws.sun.com") == -1) {
                    Object a = atts.get(cid);
                    if (a == null) {
                        a = atts.get("<" + cid + ">");
                        if (a == null) {
                            atts.put(att.getContentId(), att.asDataHandler());
                        }
                        else {
                            continue;
                        }
                    }
                    else {
                        continue;
                    }
                }
                else {
                    atts.put(att.getContentId(), att.asDataHandler());
                }
            }
            return atts;
        }
        else {
            return value;
        }
    }
    
    public void putAll(Map<? extends String, ?> t) {
        for (String key : t.keySet()) {
            if (!this.asMapIncludingInvocationProperties.containsKey(key)) {
                this.handlerScopeProps.add(key);
            }
            else {
                continue;
            }
        }
        this.asMapIncludingInvocationProperties.putAll(t);
    }
    
    public void clear() {
        this.asMapIncludingInvocationProperties.clear();
    }
    
    public Object remove(Object key) {
        this.handlerScopeProps.remove(key);
        return this.asMapIncludingInvocationProperties.remove(key);
    }
    
    public Set<String> keySet() {
        return this.asMapIncludingInvocationProperties.keySet();
    }
    
    public Set<Map.Entry<String, Object>> entrySet() {
        return this.asMapIncludingInvocationProperties.entrySet();
    }
    
    public Collection<Object> values() {
        return this.asMapIncludingInvocationProperties.values();
    }
}
